#!/usr/bin/env python

# Using the late Oregonian's pumpkin gradient

# author: Pocholo
# date: 9/5/20

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.   


from gimpfu import *

def pm_create_a_halloween_scene():

    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
			
	#Create background layer
    img = pdb.gimp_image_new(1200, 700, RGB)
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, backLayer, 0)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_drawable_fill(backLayer, FILL_FOREGROUND)
    pdb.gimp_context_set_gradient("Halloween night")
    pdb.gimp_drawable_edit_gradient_fill(backLayer, 
                                                  0,  #gradient type - linear
                                                  0,  #offset
                                               TRUE,  #super sample
                                                  3,  #supersample-max depth
                                               0.20,  #supersample-threshole
                                               TRUE,  #dither
                                                600,  #x1
                                                  0,  #y1
                                                600,  #x2
                                                700)  #y2
    
    #Process
    pdb.gimp_display_new(img)
    
    #Stars
    starsLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Stars", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, starsLayer, 0)
    pdb.gimp_drawable_fill(starsLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_drawable_fill(starsLayer, FILL_FOREGROUND)   
    pdb.plug_in_hsv_noise(img, starsLayer, 8, 0, 0, 255)
    pdb.gimp_levels(starsLayer, HISTOGRAM_VALUE, 130, 255, 1.00, 0, 255)
    pdb.plug_in_sparkle(img, starsLayer, 0.001, 0.75, 7, 4, -1, 1.00, 0, 0, 0, FALSE, FALSE, FALSE, 0)
    pdb.gimp_image_select_rectangle(img, 2, 0, 0, 1200, 500)
    pdb.gimp_selection_invert(img)
    pdb.gimp_selection_feather(img, 200)
    pdb.gimp_edit_clear(starsLayer)
    pdb.gimp_edit_clear(starsLayer)
    pdb.gimp_selection_none(img)
           
    #Create the pumpkin layer 
    size = 500
    pumLayer = pdb.gimp_layer_new(img, size, size, RGBA_IMAGE, "Pumpkin", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, pumLayer, 0)
    pdb.gimp_drawable_fill(pumLayer, FILL_TRANSPARENT)
    
    #Using a selected gradient to create the grooves of the pumpkin
    pdb.gimp_context_set_gradient("Pumpkin")
    pdb.gimp_context_set_gradient_repeat_mode(1)
    pdb.gimp_drawable_edit_gradient_fill(pumLayer, 0, 0, TRUE, 3, 0.20, TRUE, 0, 250, 20, 250)
		
    #using map to create the pumpkin shape
    pdb.plug_in_map_object(img, pumLayer, 
                                       1,    # Type of mapping (0=plane,1=sphere,2=box,3=cylinder)
                           0.5, 0.5, 2.0,    # viewpoint
                           0.5, 0.5, 0.0,    # object pos
                           1.0, 0.0, 0.0,    # first axis
                           0.0, 1.0, 0.0,    # 2nd axis
                          0.0, 0.0, 20.0,    # axis rotation
                                       0,    # Type of lightsource (0=point,1=directional,2=none)
                         (255, 255, 255),    # Lightsource color (r,g,b)
                           0.5, 0.1, 2.0,    # light position
                         -1.0, -1.0, 1.0,    # light direction
             0.80, 0.70, 0.20, 0.0, 27.0,    # material (amb, diff, refl, spec, high)
                                    True,    # antialias
                                   False,    # tile
                                   False,    # new image
                                    True,    # transparency
                                    0.25,    # Sphere/cylinder radius (only used when maptype=1 or 3)
                           0.5, 0.5, 0.5,    # box size
                                    1.00,    # unused parameter Cylinder length
                                    None,    # Box front face (set these to -1 if not used)
                                    None,    # Box back face
                                    None,    # Box top face									
                                    None,    # Box bottom face
                                    None,    # Box left face
                                    None,    # Box right face
                                    None,    # Cylinder top face (unused parameter)
                                    None)    # Cylinder bottom face (unused parameter)

    #Use g'mic spherize filter
    pdb.plug_in_gmic_qt(img, pumLayer, 1, 0, "-v - -fx_spherize 65,0.6,0,50,50,0.25,0,0,0") 
    pdb.gimp_image_select_item(img, 2, pumLayer)
    pdb.gimp_selection_shrink(img, 2)
    pdb.gimp_selection_feather(img, 7)						
    pdb.gimp_selection_invert(img)
    pdb.gimp_edit_clear(pumLayer)
    pdb.gimp_edit_clear(pumLayer)
    pdb.gimp_selection_none(img)
    pdb.gimp_levels(pumLayer, 
					        4,    #HISTOGRAM-ALPHA
						  200,    #low_input 
						  255,    #high_input 						 
						  1.5,    #gamma 
						    0,    #low_output 
						  255)    #high_output 
						  
    #The skin fiber
    pdb.gimp_image_select_item(img, 2, pumLayer)
    fiberLayer = pdb.gimp_layer_new(img, size, size, RGBA_IMAGE, "Fiber", 100, LAYER_MODE_OVERLAY)
    pdb.gimp_image_add_layer(img, fiberLayer, 0)
    pdb.gimp_drawable_fill(fiberLayer, FILL_TRANSPARENT)		
    pdb.gimp_context_set_pattern("Fibers")
    pdb.gimp_edit_bucket_fill(fiberLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.plug_in_mblur(img, fiberLayer, 0, 15, 90, 250, 250)
    pdb.plug_in_mblur(img, fiberLayer, 2, 10, 0, 250, 250)
    pdb.gimp_selection_none(img)
    
    #The pumpkin stem
    stemLayer = pdb.gimp_layer_new(img, size, size, RGBA_IMAGE, "Crease", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, stemLayer, 2)
    pdb.gimp_drawable_fill(stemLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("Stem")
    pdb.gimp_context_set_brush_size(150)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_dynamics("Dynamics Off")
    pdb.gimp_context_set_paint_method("gimp-paintbrush")
    pdb.gimp_paintbrush_default(stemLayer, 2, [260, 130])				
            
    #Create the the pumpkin face
    faceLayer = pdb.gimp_layer_new(img, size, size, RGBA_IMAGE, "Pumpkin face", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, faceLayer, 0)
    pdb.gimp_drawable_fill(faceLayer, FILL_TRANSPARENT)	
    pdb.gimp_context_set_brush("Pumpkin face ")
    pdb.gimp_context_set_brush_size(300)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_paintbrush_default(faceLayer, 2, [250, 270])
      
    #Drop Shadow
    pdb.python_layerfx_drop_shadow(img, faceLayer,
									  (74, 65, 0),   #color
											  100,   #opacity
												0,   #contou - linear
											    0,   #noise
											   16,   #mode - grain merge
											   80,   #spread
											   20,   #size
											  120,   #offset_angle
											    1,   #offset_distance
											 TRUE,   #knockout
											FALSE)   #merge
											 
    #Inner Shadow
    pdb.python_layerfx_inner_shadow(img, faceLayer, 
								         (0, 0, 0),   #color
									            30,   #opacity
										         0,   #contou - linear
										         0,   #noise
										        21,   #mode - grain merge
										         1,   #source
										        35,   #choke 
									            60,   #size 
										       -90,   #offset_angle 
										         1,   #offset_distance 
									         FALSE)  #merge									 
	 
    #Create the pumpkin inner candle glow
    #Gradient overlay
    pdb.python_layerfx_gradient_overlay(img, faceLayer, 
						                 "Candlelight",   #gradient 
									                 2,   #gradient_type - radial 
									                 0,   #repeat none 
								                  TRUE,   #reverse 
									               100,   #opacity 
									                 0,   #mode - normal 
									               250,   #center_x 
									               260,   #center_y 
									                90,   #angle 
								                   400,   #width 
								                 FALSE)   #merge
    
    #Make the inside of the pumpkin
    inLayer = pdb.gimp_layer_new(img, size, size, RGBA_IMAGE, "Inner pumpkin", 100, LAYER_MODE_OVERLAY)
    pdb.gimp_image_add_layer(img, inLayer, 0)
    pdb.gimp_drawable_fill(inLayer, FILL_TRANSPARENT)		
    pdb.gimp_image_select_item(img, 2, faceLayer)
    pdb.gimp_context_set_pattern("Stone")
    pdb.gimp_edit_bucket_fill(inLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.plug_in_mblur(img, inLayer, 0, 8, 90, 250, 250)
    pdb.plug_in_mblur(img, inLayer, 2, 20, 0, 250, 250)
    pdb.gimp_selection_none(img)
    
    #Merge the pumpkin efects layers into one
    pdb.gimp_item_set_visible(backLayer, FALSE)
    pdb.gimp_item_set_visible(starsLayer, FALSE)
    pumLayer = pdb.gimp_image_merge_visible_layers(img, CLIP_TO_BOTTOM_LAYER )
    pdb.gimp_item_set_name(pumLayer, "Pumpkin") 
    pumLayer = pdb.gimp_image_get_layer_by_name(img, "Pumpkin")
    pdb.gimp_item_set_visible(backLayer, TRUE)
    pdb.gimp_item_set_visible(starsLayer, TRUE)
    
    #Set the pumpkin position
    pumLayer = pdb.gimp_item_transform_translate(pumLayer, 0, 250)
        
    #Inner glow
    pdb.python_layerfx_inner_glow(img, pumLayer, 
							          ("Black"),   #color
											 75,   #opacity
											  0,   #contou
											  0,   #noise
								              5,   #mode - overlay
											  1,   #source
											  0,   #choke 
											 24,   #size 
										   TRUE)   #merge
    
    drawable = pdb.gimp_image_get_layer_by_name(img, "Pumpkin")
    pdb.gimp_layer_resize_to_image_size(drawable)
           
    #Moon
    moonLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Moon", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, moonLayer, 1)
    pdb.gimp_drawable_fill(moonLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("Full moon")
    pdb.gimp_context_set_brush_size(475)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_paint_method("gimp-paintbrush")
    pdb.gimp_paintbrush_default(moonLayer, 2, [620, 260])
    
    #Moon glow
    non_empty = pdb.gimp_edit_copy(moonLayer)
    glowLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Moon Glow", 100, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_add_layer(img, glowLayer, 2)
    pdb.gimp_drawable_fill(glowLayer, FILL_TRANSPARENT)
    floating_sel = pdb.gimp_edit_paste(glowLayer, FALSE)
    pdb.gimp_floating_sel_anchor(floating_sel)
    glowLayer = pdb.gimp_image_get_layer_by_name(img, "Moon Glow")
    pdb.plug_in_gauss(img, glowLayer, 100, 100, 1)
    
    #Mistery manor
    houseLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Mistery Manor", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, houseLayer, 1)
    pdb.gimp_drawable_fill(houseLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("House")
    pdb.gimp_context_set_brush_size(1000)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_paint_method("gimp-paintbrush")
    pdb.gimp_paintbrush_default(houseLayer, 2, [810, 450])
       
    #Cemetery   
    graveLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Graveyard", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, graveLayer, 1)
    pdb.gimp_drawable_fill(graveLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("Cemetery")
    pdb.gimp_context_set_brush_size(1400)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_paint_method("gimp-paintbrush")
    pdb.gimp_paintbrush_default(graveLayer, 2, [590, 580])
    
    #Create the fog
    fogLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Fog", 50, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_add_layer(img, fogLayer, 1)
    pdb.gimp_drawable_fill(fogLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_image_select_rectangle(img, 2, 0, 400, 1200, 180)   
    pdb.gimp_edit_bucket_fill(fogLayer,  
                        BUCKET_FILL_FG,  #fill_mode
                     LAYER_MODE_NORMAL,  #paint_mode
                                   100,  #opacity
                                   255,  #threshold
                                 FALSE,  #sample_merged
                                     0,  #x
                                     0)  #y

    pdb.plug_in_solid_noise(img, fogLayer, FALSE, FALSE, 1, 9, 6, 6)
    pdb.plug_in_colortoalpha(img, fogLayer, ("Black"))
    pdb.gimp_selection_none(img)
    pdb.plug_in_ripple(img, fogLayer, 200, 5, 1, 1, 1, TRUE, TRUE)
    pdb.plug_in_gauss(img, fogLayer, 50, 50, 1)
        
    #Create a wooden deck
    deckLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Deck", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, deckLayer, 1)
    pdb.gimp_drawable_fill(deckLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_rectangle(img, 2, 0, 540, 1200, 145)    
    pdb.gimp_context_set_pattern("Pine")
    pdb.gimp_edit_bucket_fill(deckLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    deckLayer = pdb.gimp_item_transform_perspective(deckLayer, 0, 540, 1200, 540, -170, 685, 1364, 685)
    pdb.gimp_floating_sel_anchor(deckLayer)
    deckLayer = pdb.gimp_image_get_layer_by_name(img, "Deck")
    pdb.gimp_layer_resize_to_image_size(deckLayer)
    pdb.script_fu_3d_extrude(img, deckLayer, 
                                          0,    #Extrusion Type
                                         15,    #Extrusion Depth
                                         90,    #Extrusion Direction
                                         90,    #Bump Azimuth
                                         45,    #Bump Elevation
                                          1,    #Bump Depth
                                       0.30,    #Bump Ambient Light
                                          0,    #Bump Curve
                                      FALSE,    #Invert Bump
                                       TRUE,    #Merge Extrusion
                                      FALSE,    #Overlay With Pattern
                                      FALSE,    #Extrude With Pattern
                                "Dried mud")    #Extrusion Pattern
    
    deckLayer = pdb.gimp_image_get_layer_by_name(img, "Extrusion")
    pdb.gimp_item_set_name(deckLayer, "Deck")
    pdb.gimp_layer_resize_to_image_size(deckLayer)

    #Candlelight glare
    glareLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Glare", 50, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_add_layer(img, glareLayer, 1)
    pdb.gimp_drawable_fill(glareLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_image_select_ellipse(img, CHANNEL_OP_REPLACE, 120, 630, 220, 40)
    pdb.gimp_edit_bucket_fill(glareLayer,  
                          BUCKET_FILL_FG,  #fill_mode
                       LAYER_MODE_NORMAL,  #paint_mode
                                     100,  #opacity
                                     255,  #threshold
                                   FALSE,  #sample_merged
                                       0,  #x
                                       0)  #y
    pdb.gimp_context_set_gradient("Candlelight")
    pdb.gimp_context_set_gradient_reverse(TRUE)
    pdb.gimp_drawable_edit_gradient_fill(glareLayer, 
                                                  2,  #gradient type - radial
                                                  0,  #offset
                                               TRUE,  #super sample
                                                  3,  #supersample-max depth
                                               0.20,  #supersample-threshole
                                               TRUE,  #dither
                                                230,  #x1
                                                650,  #y1
                                                380,  #x2
                                                650)  #y2
    pdb.gimp_selection_none(img)
    pdb.plug_in_gauss(img, glareLayer, 50, 50, 1)
    
    #Pumpkin shadow
    shadowLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Perspective Shadow", 80, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_add_layer(img, shadowLayer, 1)
    pdb.gimp_drawable_fill(shadowLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_image_select_ellipse(img, CHANNEL_OP_REPLACE, 82, 525, 330, 100)
    pdb.gimp_edit_bucket_fill(shadowLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    perpshaLayer = pdb.gimp_image_get_layer_by_name(img, "Perspective Shadow")
    pdb.gimp_selection_none(img)
    pdb.plug_in_gauss(img, shadowLayer, 3, 3, 1)
    
    #A few bats
    batsLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Bats", 100, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_add_layer(img, batsLayer, 0)
    pdb.gimp_drawable_fill(batsLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("Bats")
    pdb.gimp_context_set_brush_size(200)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_paint_method("gimp-paintbrush")
    pdb.gimp_paintbrush_default(batsLayer, 2, [650, 100])

    #Make active the bottom layer and done
    pdb.gimp_image_set_active_layer(img, backLayer) 
    #Set the defaults
    pdb.gimp_context_pop()
    
register(
    "pm_create_a_halloween_scene",
    "Creates a carved Halloween pumpkin",
    "Creates a Holloween Scene",
    "Pocholo",
    "Pocholo",
    "2020",
    "Create a Halloween Scene",
    "",
    [	
    ],
    [],
pm_create_a_halloween_scene, menu="<Image>/Python-Fu/Halloween")
main()								  